"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPos = void 0;
/**
 * Get image size according to the selected rules.
 * Note that the resources must be loaded before applying sizes to them.
 */
function getPos(data) {
    // get container size
    var containerSizes = getContainerSizes(data);
    // get source size
    var sourceSizes = getSourceSizes(data);
    // source coordinates
    var coords;
    // calculate position
    if (data.rule === 'cover') {
        coords = getPosCover(data, containerSizes, sourceSizes);
    }
    else if (data.rule === 'contain') {
        coords = getPosContain(data, containerSizes, sourceSizes);
    }
    else if (data.rule === 'top-left') {
        coords = getPosTopLeft(data, sourceSizes);
    }
    else if (data.rule === 'top-right') {
        coords = getPosTopRight(data, containerSizes, sourceSizes);
    }
    else if (data.rule === 'bottom-left') {
        coords = getPosBottomLeft(data, containerSizes, sourceSizes);
    }
    else if (data.rule === 'bottom-right') {
        coords = getPosBottomRight(data, containerSizes, sourceSizes);
    }
    else if (data.rule === 'center') {
        coords = getPosCenter(data, containerSizes, sourceSizes);
    }
    // return position
    return Object.assign(coords, {
        sourceWidth: sourceSizes.width,
        sourceHeight: sourceSizes.height,
    });
}
exports.getPos = getPos;
/**
 * Get scale prop
 * @ignore
 */
function getScale(data) {
    if (typeof data.scale !== 'undefined') {
        return data.scale;
    }
    return 1;
}
/**
 * Get source sizes
 * @ignore
 */
function getSourceSizes(data) {
    var source = data.source;
    if (!!data.sourceWidth && !!data.sourceHeight) {
        return {
            width: data.sourceWidth,
            height: data.sourceHeight,
        };
    }
    if (source instanceof HTMLVideoElement) {
        return {
            width: source.videoWidth,
            height: source.videoHeight,
        };
    }
    if (source instanceof HTMLImageElement) {
        return {
            width: source.naturalWidth,
            height: source.naturalHeight,
        };
    }
    return {
        width: source.width,
        height: source.height,
    };
}
/**
 * Get container sizes
 * @ignore
 */
function getContainerSizes(data) {
    // if has a container
    if ('container' in data) {
        return {
            width: data.container.clientWidth,
            height: data.container.clientHeight,
        };
    }
    return {
        width: data.width,
        height: data.height,
    };
}
/**
 * Get "cover" position
 * @ignore
 */
function getPosCover(data, containerSizes, sourceSizes) {
    var width = containerSizes.width * getScale(data);
    var height = sourceSizes.height * width / sourceSizes.width;
    if (height / getScale(data) < containerSizes.height) {
        height = containerSizes.height * getScale(data);
        width = sourceSizes.width * height / sourceSizes.height;
    }
    var x = (containerSizes.width - width) / 2;
    var y = (containerSizes.height - height) / 2;
    return {
        width: width,
        height: height,
        x: x,
        y: y,
    };
}
/**
 * Get "contain" position
 * @ignore
 */
function getPosContain(data, containerSizes, sourceSizes) {
    var width = 0;
    var height = 0;
    var widthRatio = sourceSizes.width / sourceSizes.height;
    if (containerSizes.width > containerSizes.height) {
        height = containerSizes.height;
        width = height * widthRatio;
        if (width > containerSizes.width) {
            width = containerSizes.width;
            height = width / widthRatio;
        }
    }
    else if (containerSizes.height >= containerSizes.width) {
        width = containerSizes.width;
        height = width / widthRatio;
        if (height > containerSizes.height) {
            height = containerSizes.height;
            width = height * widthRatio;
        }
    }
    width *= getScale(data);
    height *= getScale(data);
    var x = (containerSizes.width - width) / 2;
    var y = (containerSizes.height - height) / 2;
    return {
        width: width,
        height: height,
        x: x,
        y: y,
    };
}
/**
 * Get "top-left" position
 * @ignore
 */
function getPosTopLeft(data, sourceSizes) {
    var width = sourceSizes.width * getScale(data);
    var height = sourceSizes.height * getScale(data);
    return {
        width: width,
        height: height,
        x: 0,
        y: 0,
    };
}
/**
 * Get "top-right" position
 * @ignore
 */
function getPosTopRight(data, containerSizes, sourceSizes) {
    var width = sourceSizes.width * getScale(data);
    var height = sourceSizes.height * getScale(data);
    var x = containerSizes.width - sourceSizes.width;
    return {
        width: width,
        height: height,
        x: x,
        y: 0,
    };
}
/**
 * Get "bottom-left" position
 * @ignore
 */
function getPosBottomLeft(data, containerSizes, sourceSizes) {
    var width = sourceSizes.width * getScale(data);
    var height = sourceSizes.height * getScale(data);
    var y = containerSizes.height - sourceSizes.height;
    return {
        width: width,
        height: height,
        x: 0,
        y: y,
    };
}
/**
 * Get "bottom-right" position
 * @ignore
 */
function getPosBottomRight(data, containerSizes, sourceSizes) {
    var width = sourceSizes.width * getScale(data);
    var height = sourceSizes.height * getScale(data);
    var x = containerSizes.width - sourceSizes.width;
    var y = containerSizes.height - sourceSizes.height;
    return {
        width: width,
        height: height,
        x: x,
        y: y,
    };
}
/**
 * Get "center" position
 * @ignore
 */
function getPosCenter(data, containerSizes, sourceSizes) {
    var width = sourceSizes.width * getScale(data);
    var height = sourceSizes.height * getScale(data);
    var x = (containerSizes.width - sourceSizes.width) / 2;
    var y = (containerSizes.height - sourceSizes.height) / 2;
    return {
        width: width,
        height: height,
        x: x,
        y: y,
    };
}
