/**
 * Get image size according to the selected rules.
 * Note that the resources must be loaded before applying sizes to them.
 */
export function getPos(data) {
    // get container size
    const containerSizes = getContainerSizes(data);
    // get source size
    const sourceSizes = getSourceSizes(data);
    // source coordinates
    let coords;
    // calculate position
    if (data.rule === 'cover') {
        coords = getPosCover(data, containerSizes, sourceSizes);
    }
    else if (data.rule === 'contain') {
        coords = getPosContain(data, containerSizes, sourceSizes);
    }
    else if (data.rule === 'top-left') {
        coords = getPosTopLeft(data, sourceSizes);
    }
    else if (data.rule === 'top-right') {
        coords = getPosTopRight(data, containerSizes, sourceSizes);
    }
    else if (data.rule === 'bottom-left') {
        coords = getPosBottomLeft(data, containerSizes, sourceSizes);
    }
    else if (data.rule === 'bottom-right') {
        coords = getPosBottomRight(data, containerSizes, sourceSizes);
    }
    else if (data.rule === 'center') {
        coords = getPosCenter(data, containerSizes, sourceSizes);
    }
    // return position
    return Object.assign(coords, {
        sourceWidth: sourceSizes.width,
        sourceHeight: sourceSizes.height,
    });
}
/**
 * Get scale prop
 * @ignore
 */
function getScale(data) {
    if (typeof data.scale !== 'undefined') {
        return data.scale;
    }
    return 1;
}
/**
 * Get source sizes
 * @ignore
 */
function getSourceSizes(data) {
    const { source } = data;
    if (!!data.sourceWidth && !!data.sourceHeight) {
        return {
            width: data.sourceWidth,
            height: data.sourceHeight,
        };
    }
    if (source instanceof HTMLVideoElement) {
        return {
            width: source.videoWidth,
            height: source.videoHeight,
        };
    }
    if (source instanceof HTMLImageElement) {
        return {
            width: source.naturalWidth,
            height: source.naturalHeight,
        };
    }
    return {
        width: source.width,
        height: source.height,
    };
}
/**
 * Get container sizes
 * @ignore
 */
function getContainerSizes(data) {
    // if has a container
    if ('container' in data) {
        return {
            width: data.container.clientWidth,
            height: data.container.clientHeight,
        };
    }
    return {
        width: data.width,
        height: data.height,
    };
}
/**
 * Get "cover" position
 * @ignore
 */
function getPosCover(data, containerSizes, sourceSizes) {
    let width = containerSizes.width * getScale(data);
    let height = sourceSizes.height * width / sourceSizes.width;
    if (height / getScale(data) < containerSizes.height) {
        height = containerSizes.height * getScale(data);
        width = sourceSizes.width * height / sourceSizes.height;
    }
    const x = (containerSizes.width - width) / 2;
    const y = (containerSizes.height - height) / 2;
    return {
        width,
        height,
        x,
        y,
    };
}
/**
 * Get "contain" position
 * @ignore
 */
function getPosContain(data, containerSizes, sourceSizes) {
    let width = 0;
    let height = 0;
    const widthRatio = sourceSizes.width / sourceSizes.height;
    if (containerSizes.width > containerSizes.height) {
        height = containerSizes.height;
        width = height * widthRatio;
        if (width > containerSizes.width) {
            width = containerSizes.width;
            height = width / widthRatio;
        }
    }
    else if (containerSizes.height >= containerSizes.width) {
        width = containerSizes.width;
        height = width / widthRatio;
        if (height > containerSizes.height) {
            height = containerSizes.height;
            width = height * widthRatio;
        }
    }
    width *= getScale(data);
    height *= getScale(data);
    const x = (containerSizes.width - width) / 2;
    const y = (containerSizes.height - height) / 2;
    return {
        width,
        height,
        x,
        y,
    };
}
/**
 * Get "top-left" position
 * @ignore
 */
function getPosTopLeft(data, sourceSizes) {
    const width = sourceSizes.width * getScale(data);
    const height = sourceSizes.height * getScale(data);
    return {
        width,
        height,
        x: 0,
        y: 0,
    };
}
/**
 * Get "top-right" position
 * @ignore
 */
function getPosTopRight(data, containerSizes, sourceSizes) {
    const width = sourceSizes.width * getScale(data);
    const height = sourceSizes.height * getScale(data);
    const x = containerSizes.width - sourceSizes.width;
    return {
        width,
        height,
        x,
        y: 0,
    };
}
/**
 * Get "bottom-left" position
 * @ignore
 */
function getPosBottomLeft(data, containerSizes, sourceSizes) {
    const width = sourceSizes.width * getScale(data);
    const height = sourceSizes.height * getScale(data);
    const y = containerSizes.height - sourceSizes.height;
    return {
        width,
        height,
        x: 0,
        y,
    };
}
/**
 * Get "bottom-right" position
 * @ignore
 */
function getPosBottomRight(data, containerSizes, sourceSizes) {
    const width = sourceSizes.width * getScale(data);
    const height = sourceSizes.height * getScale(data);
    const x = containerSizes.width - sourceSizes.width;
    const y = containerSizes.height - sourceSizes.height;
    return {
        width,
        height,
        x,
        y,
    };
}
/**
 * Get "center" position
 * @ignore
 */
function getPosCenter(data, containerSizes, sourceSizes) {
    const width = sourceSizes.width * getScale(data);
    const height = sourceSizes.height * getScale(data);
    const x = (containerSizes.width - sourceSizes.width) / 2;
    const y = (containerSizes.height - sourceSizes.height) / 2;
    return {
        width,
        height,
        x,
        y,
    };
}
